package org.ibex.mail;
// todo: periodically flush out old graylist entries

import java.sql.*;
import java.net.*;
import java.io.*;
import java.util.*;
import java.sql.Timestamp;

public class Graylist extends SqliteTable {

    public Graylist(String filename) {
        super(filename,
              new String[] {
                  "create table if not exists 'whitelist' (ip unique)",
                  "create table if not exists 'graylist' (ip,fromaddr,toaddr,date, primary key(ip,fromaddr,toaddr))"
              });
        reap("graylist", "date");
    }

    public synchronized void addWhitelist(InetAddress ip) {
        try {
            PreparedStatement add    = conn.prepareStatement("insert or replace into 'whitelist' values(?)");
            add.setString(1, ip.getHostAddress());
            add.executeUpdate();
        } catch (SQLException e) { throw new RuntimeException(e); }
    }

    public synchronized boolean isWhitelisted(InetAddress ip) {
        try {
            PreparedStatement check = conn.prepareStatement("select * from 'whitelist' where ip=?");
            check.setString(1, ip.getHostAddress());
            ResultSet rs = check.executeQuery();
            return !rs.isAfterLast();
        } catch (SQLException e) { throw new RuntimeException(e); }
    }

    public synchronized long getGrayListTimestamp(InetAddress ip, String from, String to) {
        try {
            PreparedStatement check =
                conn.prepareStatement("select date from graylist where ip=? and fromaddr=? and toaddr=?");
            check.setString(1, ip.getHostAddress());
            check.setString(2, from);
            check.setString(3, to);
            ResultSet rs = check.executeQuery();
            if (rs.isAfterLast()) return 0;
            return rs.getTimestamp(1).getTime();
        } catch (SQLException e) { throw new RuntimeException(e); }
    }

    public synchronized void setGrayListTimestamp(InetAddress ip, String from, String to, long date) {
        try {
            PreparedStatement check =
                conn.prepareStatement("insert or replace into graylist (ip,fromaddr,toaddr,date) values(?,?,?,?)");
            check.setString(1, ip.getHostAddress());
            check.setString(2, from);
            check.setString(3, to);
            check.setTimestamp(4, new Timestamp(date));
            check.executeUpdate();
        } catch (SQLException e) { throw new RuntimeException(e); }
    }

}

